package cc.smtweb.framework.core.db;

import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.vo.ModelLinkName;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * Created by Akmm at 2022/5/6 15:43
 * 实体辅助类
 */
public class EntityHelper {
    //获取字段别名
    private static String getFieldAlias(Map<String, String> mapFieldAlias, String field) {
        return (mapFieldAlias != null && mapFieldAlias.containsKey(field)) ? mapFieldAlias.get(field) : field;
    }

    /**
     * 添加关联字段值
     * @param tableName
     * @param bean
     */
    public static void loadBeanLink(String tableName, SwMap bean, Map<String, String> mapFieldAlias) {
        //添加关联字段值
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table == null) return;
        List<ModelLinkName> listLink = table.findLinkeNames();
        if (listLink.isEmpty()) return;

        //有缓存的，从缓存拿，无缓存的放Map，最后从数据库去拿
        Map<String, List<String>> mapIds = new HashMap<>();

        for (ModelLinkName l : listLink) {
            final String fieldName = getFieldAlias(mapFieldAlias, l.getFieldName());
            String value = bean.readString(fieldName);
            if (StringUtils.isEmpty(value)) continue;
            String[] ids = StringUtils.split(value, ",");
            if (l.getLinkTable().isNeedCache()) {
                EntityDao dao = DbEngine.getInstance().findDao(l.getLinkTable().getName());
                AbstractCache cache = CacheManager.getIntance().getCache(l.getLinkTable().getName());
                String names = "";
                for (String sId : ids) {
                    Object b = cache.get(sId);
                    String sn = (String) dao.readValue(b, l.getLinkNameField());
                    if (StringUtils.isNotEmpty(sn)) names += "," + sn;
                }
                bean.put(fieldName + "_text", names.length() > 1 ? names.substring(1) : "");
            } else {
                List<String> list = mapIds.computeIfAbsent(l.getLinkTable().getName(), k -> new ArrayList<>());
                Collections.addAll(list, ids);
            }
        }


        if (mapIds.isEmpty()) return;
        //数据库查询
        Map<String, Map<String, String>> mapValue = new HashMap<>();
        for (Map.Entry<String, List<String>> entry : mapIds.entrySet()) {
            EntityDao dao = DbEngine.getInstance().findDao(entry.getKey());
            mapValue.put(entry.getKey(), dao.queryNames(entry.getValue()));
        }
        //加值

        for (ModelLinkName l : listLink) {
            if (!mapValue.containsKey(l.getLinkTable().getName())) continue;
            Map<String, String> mapV = mapValue.get(l.getLinkTable().getName());

            final String fieldName = getFieldAlias(mapFieldAlias, l.getFieldName());
            String value = bean.readString(fieldName);
            if (StringUtils.isEmpty(value)) continue;

            String[] ids = StringUtils.split(value, ",");
            String names = "";
            for (String sId : ids) {
                String sn = mapV.get(sId);
                if (StringUtils.isNotEmpty(sn)) names += "," + sn;
            }
            bean.put(fieldName + "_text", names.substring(1));
        }
    }

    public static void loadBeanLink(String tableName, List<SwMap> listData, Map<String, String> mapFieldAlias) {
        //添加关联字段值
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table == null) return;
        List<ModelLinkName> listLink = table.findLinkeNames();
        if (listLink.isEmpty()) return;

        //有缓存的，从缓存拿，无缓存的放Map，最后从数据库去拿
        Map<String, List<Long>> mapIds = new HashMap<>();
        for (SwMap row : listData) {
            for (ModelLinkName l : listLink) {
                final String fieldName = getFieldAlias(mapFieldAlias, l.getFieldName());
                String value = row.readString(fieldName);
                if (StringUtils.isEmpty(value)) continue;
                String[] ids = StringUtils.split(value, ",");
                if (l.getLinkTable().isNeedCache()) {
                    EntityDao dao = DbEngine.getInstance().findDao(l.getLinkTable().getName());
                    AbstractCache cache = CacheManager.getIntance().getCache(l.getLinkTable().getName());
                    String names = "";
                    for (String sId : ids) {
                        Object b = cache.get(sId);
                        String sn = (String) dao.readValue(b, l.getLinkNameField());
                        if (StringUtils.isNotEmpty(sn)) names += "," + sn;
                    }
                    if (StringUtils.isNotEmpty(names)) {
                        row.put(fieldName + "_text", names.substring(1));
                    } else {
                        row.put(fieldName + "_text", "");
                    }
                } else {
                    List<Long> list = mapIds.computeIfAbsent(l.getLinkTable().getName(), k -> new ArrayList<>());
                    for (String id : ids) {
                        list.add(Long.parseLong(id));
                    }
                }
            }
        }

        if (mapIds.isEmpty()) return;
        //数据库查询
        Map<String, Map<String, String>> mapValue = new HashMap<>();
        for (Map.Entry<String, List<Long>> entry : mapIds.entrySet()) {
            EntityDao dao = DbEngine.getInstance().findDao(entry.getKey());
            mapValue.put(entry.getKey(), dao.queryNames(entry.getValue()));
        }
        //加值
        for (SwMap row : listData) {
            for (ModelLinkName l : listLink) {
                if (!mapValue.containsKey(l.getLinkTable().getName())) continue;
                Map<String, String> mapV = mapValue.get(l.getLinkTable().getName());

                final String fieldName = getFieldAlias(mapFieldAlias, l.getFieldName());
                String value = row.readString(fieldName);
                if (StringUtils.isEmpty(value)) continue;

                String[] ids = StringUtils.split(value, ",");
                String names = "";
                for (String sId : ids) {
                    String sn = mapV.get(sId);
                    if (StringUtils.isNotEmpty(sn)) names += "," + sn;
                }
                row.put(fieldName + "_text", names.substring(1));
            }
        }
    }
}
