package cc.smtweb.framework.core.mvc.service;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwException;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.jdbc.AbsDbWorker;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelIndex;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

/**
 * Created by Akmm at 2022/3/2 19:52
 * 默认实体实现
 */
public class DefaultSaveHandler<T extends DefaultEntity> extends AbstractSaveHandler<T> {
    protected String tableName;

    private List<T> listTreeBean = null;

    public DefaultSaveHandler(String tableName) {
        this.tableName = tableName;
    }

    @Override
    protected T createComp() {
        final EntityDao<T> dao = DbEngine.getInstance().findDao(tableName);
        T bean = dao.createBean();
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        ModelField field = table.findFieldByType(SwEnum.FieldType.CREATE_USER.value);
        if (field != null) bean.put(field.getName(), us.getUserId());
        field = table.findFieldByType(SwEnum.FieldType.UPDATE_USER.value);
        if (field != null) bean.put(field.getName(), us.getUserId());
        return bean;
    }

    @Override
    protected long readId() {
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        return params.readLong(table.getIdField());
    }

    @Override
    protected void setNewId() {
        bean.setEntityId(DbEngine.getInstance().nextId());
    }

    @Override
    protected T loadComp(long id) {
        return new DefaultProvider<T>(tableName).getBean(id);
    }

    @Override
    protected void readFromPage() {
        bean.readFromMap(params);
    }

    @Override
    protected void checkValid()  {
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        for (ModelField field : table.getFields()) {
            if (field.isNotNull() && StringUtils.isEmpty(bean.getStr(field.getName()))) {
                throw new SwException(field.getTitle() + "不能为空！");
            }
            if (field.getFieldType() == SwEnum.FieldType.PARENT_ID.value) {
                TreeHelper.getTreeHelper(tableName).checkParent(bean);
            }
        }

        EntityDao dao = DbEngine.getInstance().findDao(tableName);
        for (ModelIndex mi : table.getIndexes()) {
            if (mi.isUnique()) {
                dao.checkUnique(bean, mi.getFields().split(","));
            }
        }
    }

    @Override
    protected void saveDb()  {
        EntityDao<T> dao = DbEngine.getInstance().findDao(tableName);
        if (isNew) {
            dao.insertEntity(bean);
        } else {
            ModelTable table = ModelTableCache.getInstance().getByName(tableName);
            ModelField field = table.findFieldByType(SwEnum.FieldType.UPDATE_USER.value);
            if (field != null) bean.put(field.getName(), us.getUserId());

            updateBean(dao);
            if (table.getType() == SwEnum.TableType.TYPE_TREE.value) {
                listTreeBean = TreeHelper.getTreeHelper(tableName).resetTreeLevel(bean);
            }
        }
    }

    //更新入库
    protected void updateBean(EntityDao<T> dao) {
        dao.updateEntity(bean);
    }


    @Override
    protected void saveSuccess() {
        super.saveSuccess();
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table.isNeedCache()) {
            AbstractCache<T> cache = CacheManager.getIntance().getCache(tableName);
            //树型表，父亲改变了，要多处理下缓存；还有个东东：级次码
            if (listTreeBean != null && !listTreeBean.isEmpty()) {
                for (T b : listTreeBean) {
                    cache.put(b);
                }
            } else {
                cache.put(bean);
            }
        }
    }

    @Override
    protected void saveFailed() {
        super.saveFailed();
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table.isNeedCache()) {
            AbstractCache<T> cache = CacheManager.getIntance().getCache(tableName);
            cache.reset(bean);
        }
    }

    //树，改变父亲
    public R changeParent(){
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table.getType() != SwEnum.TableType.TYPE_TREE.value) throw new SwException("非树型表，不支持更改父节点");

        long id = params.readLong("id");
        long parentId = params.readLong("parent_id");

        AbstractCache<T> cache = CacheManager.getIntance().getCache(tableName);
        if (table.isNeedCache()) {
            bean = cache.get(id);
        } else {
            bean = loadComp(id);
        }
        TreeHelper.getTreeHelper(tableName).checkParent(bean);
        //不考虑没有级次码等情况，这个在表设计时校验
        ModelField fieldParentId = table.findFieldByType(SwEnum.FieldType.PARENT_ID.value);

        bean.put(fieldParentId.getName(), parentId);
        DbEngine.getInstance().doTrans(new AbsDbWorker() {
            @Override
            public void work() {
                EntityDao dao = DbEngine.getInstance().findDao(tableName);
                listTreeBean = TreeHelper.getTreeHelper(tableName).resetTreeLevel(bean);
                dao.updateEntity(bean, fieldParentId.getName());
            }

            @Override
            public void doAfterDbCommit(){
                saveSuccess();
            }

            @Override
            public void doAfterDbRollback() {
                saveFailed();
            }
        });
        return R.success();
    }
}
