package cc.smtweb.system.bpm.core.ui.bind;

import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.mvc.controller.IBeanContext;
import cc.smtweb.framework.core.mvc.controller.IEditor;
import cc.smtweb.system.bpm.core.exception.BpmException;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.IParamConst;
import cc.smtweb.system.bpm.core.ui.entity.BpmPage;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.spring.cache.BpmPageCache;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

public class BpmPageContextEditor implements IEditor {

  public BpmPageContextEditor(){
  }

  @Override
  public Object getParamValue(String paramName, Class<?> paramType, IBeanContext context, Map<String, Object> params, String body, HttpServletRequest request) {
    BpmPageContext pageContext = new BpmPageContext(context, params, body, request);
    SwMap mapBody = pageContext.getBody();
    String pageName = mapBody.readString(IParamConst.PARAM_PAGE_NAME);
    if (pageName == null) {
      pageName = mapBody.readString(IParamConst.PARAM_PAGE_ID);
    }

    BpmPage page = loadPage(pageName);
    pageContext.setPageName(page.getFullName());
    pageContext.setDeploy(page.isDeploy());
    pageContext.setDataset(findUiDataset(page, mapBody.readString(IParamConst.PARAM_DATASET)));

    return pageContext;
  }

  private BpmPage loadPage(String pageName) {
    if (pageName == null) {
      throw new BpmException("参数 pageName 必填");
    }

    BpmPage uiPage = BpmPageCache.getInstance().get(pageName);
    if (uiPage == null) {
      uiPage = BpmPageCache.getInstance().getByOtherKey(pageName);
    }

    if (uiPage == null) {
      throw new BpmException("未找到页面, name=" + pageName);
    }

    return uiPage;
  }

  private BpmDataset findUiDataset(BpmPage uiPage, String datasetName) {
    if (datasetName == null) {
      throw new BpmException("参数 dataset 必填");
    }

    BpmDataset dataset = uiPage.findDatasetByName(datasetName);

    if (dataset == null) {
      throw new BpmException("未找到dataset, pageId=" + uiPage.getId() + ", id=" + datasetName);
    }

    return dataset;
  }
}
