package cc.smtweb.system.bpm.core.ui.service;

import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.system.bpm.core.exception.BpmException;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.bind.IBpmBeanConst;
import cc.smtweb.system.bpm.util.BeanUtil;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

public abstract class AbstractDatasetAgent implements IBpmBeanConst {
  protected Class<?> clazz;
  protected boolean mapClass;

  public Class<?> getBodyClass() {
    return this.clazz;
  }

  public void checkType(Class clazz) {
    if (clazz == null) {
      return;
    }

    if (this.clazz == null) {
      this.clazz = clazz;
      this.mapClass = SwMap.class.equals(clazz);
      return;
    }

    if (!this.clazz.equals(clazz)) {
      throw new BpmException("type " + this.clazz.getName() + " is not equals " + clazz.getName());
    }
  }

//  private Object createBean(SwMap body) {
//    if (!this.mapClass) {
//      Object bean;
//      try {
//        bean = clazz.newInstance();
//      } catch (InstantiationException | IllegalAccessException e) {
//        throw new BpmException(e);
//      }
//
//      BeanUtil.mapToBean(body, bean);
//      return bean;
//    } else {
//      return body;
//    }
//  }

  private void mapToBean(SwMap body, Object bean) {
    if (!this.mapClass && bean != null) {
      BeanUtil.mapToBean(body, bean);
    }
  }

  private void beanToMap(Object bean, SwMap body) {
    if (!this.mapClass && bean != null) {
      BeanUtil.beanToMap(bean, body);
    }
  }

  private List<?> createListBean(List<SwMap> bodyList) {
    if (!this.mapClass) {
      List<Object> beans = new ArrayList<>(bodyList.size());
      try {
        for (SwMap body: bodyList) {
          Object bean = clazz.newInstance();
          BeanUtil.mapToBean(body, bean);
          beans.add(bean);
        }
      } catch (InstantiationException | IllegalAccessException e) {
        throw new BpmException(e);
      }
      return beans;
    } else {
      return bodyList;
    }
  }

  private List<SwMap> createListMap(List<?> beans) {
    List<SwMap> bodyList = null;
    if (beans != null) {
      if (!this.mapClass) {
        bodyList = new ArrayList<>(beans.size());
        for (Object bean: beans) {
          SwMap body = new SwMap();
          BeanUtil.beanToMap(bean, body);
          bodyList.add(body);
        }
      } else {
        return (List<SwMap>)beans;
      }
    }

    return bodyList;
  }

//  public static <T, V, S> Class<T> resolveArgClass(Class<V> type, Class<S> subType, int index) {
//    Class<T> clazz = null;
//    if (type != null) {
//      Class<?>[] typeArgs = TypeResolver.resolveRawArguments(type, subType);
//      if (typeArgs.length > index) {
//        clazz = (Class<T>) typeArgs[index];
//      } else {
//        throw new BpmException("resolveRawArguments error " + type.getName() + "," + subType.getName());
//      }
//    }
//    return clazz;
//  }

  protected void saveAttrRowMap(BpmPageContext pageContext, SwMap value) {
    HttpServletRequest request = pageContext.getRequest();
    if (value != null) {
      request.setAttribute(BEAN_ROW_MAP, value);
    } else {
      request.removeAttribute(BEAN_ROW_MAP);
    }
    request.removeAttribute(BEAN_ROW_BEAN);
  }

  protected void saveAttrRow(BpmPageContext pageContext, Object value) {
    if (this.mapClass) {
      saveAttrRowMap(pageContext, (SwMap)value);
    } else {
      HttpServletRequest request = pageContext.getRequest();
      if (value != null) {
        request.setAttribute(BEAN_ROW_BEAN, value);
      } else {
        request.removeAttribute(BEAN_ROW_BEAN);
      }
      request.removeAttribute(BEAN_ROW_MAP);
    }
  }

  protected SwMap loadAttrRowMap(BpmPageContext pageContext) {
    HttpServletRequest request = pageContext.getRequest();

    Object map = request.getAttribute(BEAN_ROW_MAP);
    if (map != null) {
      return (SwMap)map;
    }

    Object bean = request.getAttribute(BEAN_ROW_BEAN);
    if (bean != null) {
      SwMap body = new SwMap();
      beanToMap(bean, body);
      return body;
    }

    return null;
  }

  protected void saveAttr(BpmPageContext pageContext, String key, Object value) {
    HttpServletRequest request = pageContext.getRequest();
    if (value != null) {
      request.setAttribute(key, value);
    } else {
      request.removeAttribute(key);
    }
  }

  protected List<SwMap> loadAttrRowsMap(BpmPageContext pageContext) {
    HttpServletRequest request = pageContext.getRequest();

    Object map = request.getAttribute(BEAN_ROWS_MAP);
    if (map != null) {
      return (List<SwMap>)map;
    }

    Object bean = request.getAttribute(BEAN_ROWS_BEAN);
    if (bean != null) {
      return createListMap((List<?>)bean);
    }

    return null;
  }

  protected void saveAttrRows(BpmPageContext pageContext, List<?> value) {
    if (this.mapClass) {
      saveAttrRowsMap(pageContext, (List<SwMap>)value);
    } else {
      HttpServletRequest request = pageContext.getRequest();
      if (value != null) {
        request.setAttribute(BEAN_ROWS_BEAN, value);
      } else {
        request.removeAttribute(BEAN_ROWS_BEAN);
      }
      request.removeAttribute(BEAN_ROW_MAP);
    }
  }

  protected void saveAttrRowsMap(BpmPageContext pageContext, List<SwMap> value) {
    HttpServletRequest request = pageContext.getRequest();
    if (value != null) {
      request.setAttribute(BEAN_ROWS_MAP, value);
    } else {
      request.removeAttribute(BEAN_ROWS_MAP);
    }
    request.removeAttribute(BEAN_ROWS_BEAN);
  }
}
