package cc.smtweb.system.bpm.core.ui.service.card;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.system.bpm.core.ui.BpmKeyValue;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.IParamConst;
import cc.smtweb.system.bpm.core.ui.dataset.FieldFormatTool;
import cc.smtweb.system.bpm.core.ui.dataset.FieldFormatter;
import cc.smtweb.system.bpm.core.ui.entity.dataset.*;
import cc.smtweb.system.bpm.core.ui.service.AbstractDatasetService;
import cc.smtweb.system.bpm.engine.ui.entity.consts.MappingTypeEnum;
import cc.smtweb.system.bpm.engine.ui.entity.consts.UiEnum;
import org.apache.commons.lang3.StringUtils;

import java.util.Map;

/**
 * card数据模型处理服务抽象类
 * @author xkliu
 */
public abstract class AbstractDatasetCardService extends AbstractDatasetService {
  /**
   * 创建卡片数据集load处理器
   * @param pageContext BPM页面上小文
   * @return 列表数据集时间处理器
   */
  protected abstract DatasetCardWorker createCardLoadHandler(BpmPageContext pageContext);

  /**
   * 创建卡片数据集save处理器
   * @param pageContext BPM页面上小文
   * @return 列表数据集时间处理器
   */
  protected abstract DatasetCardWorker createCardSaveHandler(BpmPageContext pageContext);

  /**
   * 创建卡片数据集remove处理器
   * @param pageContext BPM页面上小文
   * @return 列表数据集时间处理器
   */
  protected abstract DatasetCardWorker createCardRemoveHandler(BpmPageContext pageContext);

  /**
   * 初始化Load事件调度器中的事件
   * @param bpmPageContext Page上下文
   */
//  protected abstract void setupRemove(IDatasetRemoveHandler handler);

  public R load(BpmPageContext bpmPageContext) {
    DatasetCardWorker handler = createCardLoadHandler(bpmPageContext);
//    setupLoad(handler);
    BpmDataset dataset = handler.getDataset();

    // 读取主表数据
    long id = bpmPageContext.getParamId();
    SwMap result = handler.load(dataset, id);

    // 读取关联表数据
    if (result != null && dataset.isUseLookup()) {
      for (Map.Entry<String, BpmField> entry: dataset.getFields().entrySet()) {
        String uiKey = entry.getKey();
        BpmField bpmField = entry.getValue();

        AbstractBpmFieldLookup lookup = bpmField.getLookup();
        if (lookup != null) {
          // 读取查找表数据 select in, FieldId
          // 生成ID列表查询条件
          // uiField.getId());
          Object lookupKey = result.get(uiKey);

          if (lookupKey != null) {
            BpmKeyValue lookupValue = null;
            if (lookup.type() == UiEnum.FieldLookupType.TABLE) {
              lookupValue = handler.loadLookup(dataset, (BpmFieldLookup) lookup, lookupKey);
            } else if (lookup.type() == UiEnum.FieldLookupType.DICT) {
              lookupValue = loadDict((BpmFieldDict) lookup, lookupKey);
            }

            if (lookupValue != null) {
              String textKey = uiKey + IParamConst.LOOKUP_TEXT;

              if (lookupValue.getOptValue() != null) {
                result.put(textKey, lookupValue.getOptValue());
              }

              boolean isManyToMany = lookup.getMappingType().equals(MappingTypeEnum.manyToMany);
              if (isManyToMany) {
                result.put(uiKey, lookupValue.getOptKey());
                result.put(uiKey + IParamConst.LOOKUP_KEY, lookupValue.getOptKey());
              }
            }
          }
        } else {
          // 格式化字段显示
          FieldFormatter fieldFormatter = FieldFormatTool.buildFormatter(bpmField);
          if (fieldFormatter != null) {
            Object value = result.get(fieldFormatter.getName());
            if (value != null) {
              result.put(fieldFormatter.getName() + IParamConst.LOOKUP_TEXT, fieldFormatter.format(value));
            }
          }
        }
      }
    }

    return R.success(result);
  }

  private BpmKeyValue loadDict(BpmFieldDict lookup, Object lookupKey) {
    /*if (lookupKey != null) {
      // 字典数据
      String key = lookupKey.toString();
      DictPO dictPo = dbCache.dictEntity(DictPO.class, lookup.getDictType(), key);

      if (dictPo != null) {
        BpmKeyValue item = new BpmKeyValue();
        item.setOptKey(key);
        item.setOptValue(dictPo.getDictLabel());

        return item;
      }
    }*/

    return null;
  }

  public R save(BpmPageContext bpmPageContext) {
    DatasetCardWorker handler = createCardSaveHandler(bpmPageContext);
//    setupSave(handler);
    Long id = handler.save();
    SwMap body = bpmPageContext.getBody();
    if (id != null) {
      body.put("id", id);
    }
    return R.success(body);
  }

  public R remove(BpmPageContext bpmPageContext) {
    DatasetCardWorker handler = createCardRemoveHandler(bpmPageContext);
//    setupRemove(handler);
    BpmDataset dataset = handler.getDataset();

    long dataId = bpmPageContext.getParamId();

    int data = 0;
    if (dataId != 0) {
      data = handler.remove(dataset, dataId);
    } else {
      String dataIds = bpmPageContext.getBody().readString(IParamConst.PARAMS_IDS);

      if (StringUtils.isNotBlank(dataIds)) {
        data = handler.removeList(dataset, dataIds);
      }
    }

    return R.success(data);
  }
}
