package cc.smtweb.system.bpm.core.ui.service.card.agent;

import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.service.AbstractDatasetAgent;
import cc.smtweb.system.bpm.core.ui.service.DatasetApiInvoker;
import lombok.Setter;

import java.util.function.Function;

/**
 * card数据集保存方法代理执行，调用@BpmEvent的事件
 * @author xkliu
 */
public class DatasetSaveAgent extends AbstractDatasetAgent {
  @Setter
  private DatasetApiInvoker beforeSave;
  @Setter
  private DatasetApiInvoker afterSave;
  @Setter
  private DatasetApiInvoker save;
  private final Function<BpmPageContext, Long> defaultSaveFunc;

  public DatasetSaveAgent(Function<BpmPageContext, Long> defaultSaveFunc) {
    this.defaultSaveFunc = defaultSaveFunc;
  }

  public Long doSave(BpmPageContext pageContext) {
    SwMap body = pageContext.getBody();
    saveAttrRowMap(pageContext, body);
    // bean 处理
    if (beforeSave != null) {
      beforeSave.invoke(pageContext);
    }

    Long result;
    if (this.save != null) {
      result = (Long)this.save.invoke(pageContext);
    } else {
      result = this.defaultSaveFunc.apply(pageContext);
    }

    if (afterSave != null) {
      afterSave.invoke(pageContext);
    }

    return result;
  }



}
