package cc.smtweb.system.bpm.engine.process.impl;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.system.bpm.engine.process.impl.entity.CommitTaskVO;
import cc.smtweb.system.bpm.engine.process.impl.entity.LoadTaskAckVO;
import cc.smtweb.system.bpm.engine.process.impl.entity.LoadTaskVO;
import cc.smtweb.system.bpm.engine.process.impl.entity.SaveTaskVO;
import cc.smtweb.system.bpm.engine.process.runtime.ProcessInstance;
import cc.smtweb.system.bpm.engine.process.task.Task;
import cc.smtweb.system.bpm.engine.process.vo.FlowAction;
import cc.smtweb.system.bpm.engine.process.vo.FlowNode;
import cc.smtweb.system.bpm.engine.process.vo.FlowNodeField;
import cc.smtweb.system.bpm.spring.IProcessEngine;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

// "工作流任务接口"
public abstract class AbstractTaskCardService {
  @SwParam
  protected IProcessEngine processEngine;

  @SwParam
  protected DbEngine dbEngine;

  /** 读取表单界面结构和数据 */
  protected abstract LoadTaskAckVO loadForm(Task task);

  /** 保存表单数据 */
  protected void saveForm(Task task, SaveTaskVO postTaskVO, UserSession us) {
    ProcessInstance processInstance = task.getProcessInstance();
    FlowNode flowNode = processInstance.getCurNode();
    List<FlowNodeField> fields = flowNode.getFields();

    if (fields != null) {
      ModelTable modelTable = processInstance.getModelTable();
      SwMap body = new SwMap(fields.size());
      for (FlowNodeField nodeField : fields) {
        ModelField modelField = modelTable.findField(nodeField.getId());
        int perm = nodeField.getPerm();

        if (modelField != null && perm == FlowNodeField.PERM_WRITE) {
          Object value = postTaskVO.get(modelField.getName());
          if (value != null) {
            body.put(modelField.getName(), value);
          }
        }
      }

      processEngine.getFormDataStore().save(processInstance.getId(), processInstance.getRootId(), processInstance.getRootId(),
              postTaskVO.readTaskId(), modelTable.fullName(), body, us);
    }
  }

  private Task loadTask(long flowId, long taskId, String nodeKey) {
    Task task;

    if (taskId == 0) {
      task = processEngine.newTask(flowId, nodeKey);
    } else {
      task = processEngine.loadTask(taskId);
    }

    return task;
  }

  // @ApiOperation(value = "读取界面结构")
  public R load(UserSession us, @SwBody LoadTaskVO loadTaskVO) {
//    String form = "{\"form\": [{\"component\":\"fy-text\",\"props\":{\"name\":\"用户名\",\"label\":\"用户名\",\"placeholder\":\"用户名\",\"rules\":[{\"required\":true,\"message\":\"请填写用户名\"}]}},{\"component\":\"fy-text\",\"props\":{\"type\":\"password\",\"name\":\"密码\",\"label\":\"密码\",\"placeholder\":\"密码\",\"rules\":[{\"required\":true,\"message\":\"请填写密码\"}]}}]}";

    Task task = loadTask(loadTaskVO.getFlowId(), loadTaskVO.getTaskId(), null);
    if (task == null) {
      return R.error("创建任务失败");
    }

    LoadTaskAckVO data = loadForm(task);

    if (data != null) {
      Map<String, Object> formData = data.getData();
      ProcessInstance processInstance = task.getProcessInstance();
      if (task.getCreateAt() == 0L) {
        formData.put(SaveTaskVO.FLOW_ID, processInstance.getFlow().getId());
      } else {
        formData.put(SaveTaskVO.TASK_ID, task.getId());
      }

      FlowNode curNode = processInstance.getCurNode();
      formData.put(SaveTaskVO.FLOW_NODE_ID, curNode.getId());
      formData.put("bpm_flowNodeLabel", curNode.getLabel());

      return R.success(data);
    }

    return R.error("读取表单数据出错");
  }

  //  @ApiOperation(value = "保存表单数据")
  public R save(UserSession us, @SwBody SaveTaskVO saveTaskVO) {
    long taskId = saveTaskVO.readTaskId();
    Task task = this.loadTask(saveTaskVO.readFlowId(), taskId, saveTaskVO.readFlowNodeId());

    if (task == null) {
      return R.error("加载任务失败");
    }

    task.setAssigneeUserId(us.getUserId());

    dbEngine.doTrans(() -> {
      processEngine.saveTask(task);

      // 保存数据到表
      saveForm(task, saveTaskVO, us);

      return true;
    });

    Map<String, Object> data = new HashMap<>(8);
    data.put(SaveTaskVO.FLOW_ID, task.getProcessInstance().getFlow().getId());
    data.put(SaveTaskVO.TASK_ID, task.getId());
    data.put(SaveTaskVO.TASK_LOCK_REVID, task.getLockRev());
    data.put(SaveTaskVO.FLOW_ACTION_ID, saveTaskVO.readFlowActionId());
    return R.success(data);
  }

  //  @ApiOperation(value = "有选人情况下的提交")
  public R commit(UserSession us, @SwBody CommitTaskVO commitTaskVO) {
    if (commitTaskVO.getUsers() == null || commitTaskVO.getUsers().size() == 0) {
      return R.error("执行人数据为空");
    }

    Task task = processEngine.loadTask(commitTaskVO.getTaskId());
    FlowAction flowAction = task.getProcessInstance().getCurNode().checkAndGetAction(commitTaskVO.getActionKey());

    processEngine.commitTask(task, flowAction, commitTaskVO.getUsers());

    return R.success();
  }
}
