package cc.smtweb.system.bpm.engine.ui.impl;

import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.sqlbuilder.InsertSqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.SqlBuilder;
import cc.smtweb.framework.core.db.sqlbuilder.UpdateSqlBuilder;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.DateUtil;
import cc.smtweb.framework.core.util.JsonUtil;
import cc.smtweb.system.bpm.engine.ui.IFormDataStore;
import org.apache.commons.lang3.StringUtils;

public class FormDataStoreImpl implements IFormDataStore {
  private DbEngine dbEngine;

  public FormDataStoreImpl(DbEngine dbEngine) {
    this.dbEngine = dbEngine;
  }

  @Override
  public SwMap load(Long processId, Long processRootId) {
    if (processRootId != null) {
      String content = dbEngine.queryString("select pdl_content from sw_bpm.bpm_process_detail_log where pdl_id=?", processRootId);
      if (StringUtils.isNotBlank(content)) {
        return JsonUtil.parse(content, SwMap.class);
      }
    }

    return null;
  }

  @Override
  public void save(Long processId, Long processRootId, Long taskId, Long businessKey, String tableName, SwMap body, UserSession us) {
      String content = JsonUtil.encodeString(body);

      if (processId == null) {
        processId = businessKey;
      }

      if (taskId == null || taskId == 0) {
        InsertSqlBuilder sqlBuilder = SqlBuilder.createInsert("sw_bpm.bpm_process_detail_log");
        sqlBuilder.add("pdl_id", businessKey)
                .add("pdl_process_id", processId)
                .add("pdl_process_root_id", processRootId)
                .add("pdl_table", tableName)
                .add("pdl_content", content)
                .add("pdl_create_uid", us.getUserId())
                .add("pdl_update_uid", us.getUserId());
        sqlBuilder.update(dbEngine);
      } else {
        UpdateSqlBuilder sqlBuilder = SqlBuilder.createUpdate("sw_bpm.bpm_process_detail_log");
        sqlBuilder.add("pdl_process_id", processId)
                .add("pdl_task_id", taskId)
                .add("pdl_content", content)
                .add("pdl_last_time", DateUtil.nowDateTimeLong())
                .add("pdl_update_uid", us.getUserId())
                .addWhere("pdl_id", businessKey);
        sqlBuilder.update(dbEngine);
      }
  }
}
