package cc.smtweb.system.bpm.web.design.flow;

import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.mvc.service.*;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.SqlUtil;
import cc.smtweb.system.bpm.web.design.form.ModelForm;
import cc.smtweb.system.bpm.web.design.form.ModelFormLoadHandler;
import cc.smtweb.system.bpm.web.design.form.ModelFormSaveHandler;
import org.apache.commons.lang3.StringUtils;

import java.sql.ResultSetMetaData;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Akmm at 2022/3/22 9:12
 */
@SwService
public class ModelProcService extends AbstractCompService {
    @Override
    protected AbstractHandler createHandler(String type) {
        switch (type) {
            case TYPE_LOAD:
                return new ModelFormLoadHandler();
            case TYPE_SAVE:
                return new ModelFormSaveHandler();
            case TYPE_DEL:
                return new DefaultDelHandler<ModelForm>(ModelForm.ENTITY_NAME);
            case TYPE_LIST:
                return new DefaultListHandler<ModelForm>(ModelForm.ENTITY_NAME);

        }
        return null;
    }

    //保存数据集
    public R saveDataset(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_SAVE, handler -> ((ModelFormSaveHandler)handler).saveDataset());
    }

    //加载数据集
    public R loadDataset(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_LOAD, handler -> ((ModelFormLoadHandler)handler).loadDataset());
    }

    //保存页面模型
    public R saveModel(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_SAVE, handler -> ((ModelFormSaveHandler)handler).saveModel());
    }

    //加载页面模型
    public R loadModel(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_LOAD, handler -> ((ModelFormLoadHandler)handler).loadModel());
    }

    //加载引擎用页面模型
    public R model(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_LOAD, handler -> ((ModelFormLoadHandler)handler).loadForm());
    }

    //获取页面使用的控件的filter信息
    public R loadWidgetFilter(@SwBody SwMap params, UserSession us) {
        return pageHandler(params, us, TYPE_LOAD, handler -> ((ModelFormLoadHandler)handler).loadWidgetFilter());
    }

    //获取自定义sql的字段信息，去库里查
    public R loadSqlFields(@SwBody SwMap params, UserSession us) {
        try {
            String sql = params.readString("sql");
            if (StringUtils.isEmpty(sql)) return R.error("没有传入的sql！");
            sql = sql.trim().toLowerCase();
            if (!sql.startsWith("select ")) return R.error("非查询类sql，禁止执行！");
            if (sql.contains(";")) return R.error("sql内禁止出现分号！");
            sql = SqlUtil.replaceTable(sql);
            List<SwMap> ret = DbEngine.getInstance().query(sql + " where 1=0", rs -> {
                List<SwMap> fields = new ArrayList<>();
                ResultSetMetaData metaData = rs.getMetaData();

                for (int i = 1, count = metaData.getColumnCount(); i <= count; i++) {
                    SwMap col = new SwMap(2);
                    col.put("name", metaData.getColumnLabel(i));
                    SwEnum.DataTypeBean dtb = SwEnum.DataType.getBySqlType(metaData.getColumnType(i), metaData.getPrecision(i), metaData.getScale(i));
                    col.put("dataType", dtb.value);
                    fields.add(col);
                }
                return fields;
            });
            return R.success(ret);
        } catch (Exception e) {
            return R.error("操作失败！", e);
        }
    }
}
