package cc.smtweb.system.bpm.web.design.form;

import cc.smtweb.framework.core.annotation.SwCache;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.ResultSetExtractor;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

/**
 * Created by Akmm at 2022/1/12 18:34
 */
@SwCache(ident = "ASP_MODEL_FORM", title = "页面定义")
public class ModelFormCache extends AbstractCache<ModelForm> {
    private final static String mk = "k";
    private final static String mp = "prj";
    private final static String mc = "c";

    public static ModelFormCache getInstance() {
        return CacheManager.getIntance().getCache(ModelFormCache.class);
    }

    public ModelFormCache() {
        regMap(mk, k-> k.getName().toUpperCase());
        regList(mp, k-> String.valueOf(k.getPrjId()));
        regList(mc, k-> String.valueOf(k.getMcId()));
//        regList(mf, k-> k.get);
    }

    @Override
    protected String getId(ModelForm bean) {
        return String.valueOf(bean.getId());
    }

    @Override
    protected List<ModelForm> loadAll() {
        EntityDao<ModelForm> dao = DbEngine.getInstance().findDao(ModelForm.class);
        return dao.query();
    }

    public final ModelForm getByName(String key) {
        return getByKey(mk, key.toUpperCase());
    }

    public final Set<ModelForm> getFormsByMc(long mcId) {
        return getListByKey(mc, String.valueOf(mcId));
    }

    public final List<ModelForm> getFormsByMc(long mcId, Comparator<ModelForm> comparator) {
        Set<ModelForm> set = getListByKey(mc, String.valueOf(mcId));
        if (set == null || set.isEmpty()) return null;
        List<ModelForm> list = new ArrayList<>(set);
        list.sort(comparator);
        return list;
    }
}
