package cc.smtweb.system.bpm.web.design.form;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwException;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.jdbc.AbsDbWorker;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.mvc.service.DefaultSaveHandler;
import org.apache.commons.lang3.StringUtils;

/**
 * Created by Akmm at 2022/5/9 17:05
 * 页面定义保存，注意不要覆盖content和dataset
 */
public class ModelFormSaveHandler extends DefaultSaveHandler<ModelForm> {
    public ModelFormSaveHandler() {
        super(ModelForm.ENTITY_NAME);
    }

    @Override
    protected void updateBean(EntityDao<ModelForm> dao) {
        dao.updateEntityEx(bean, "mf_content", "mf_dataset");
    }

    public R saveDataset() {
        long id = params.readLong("id");
        String data = params.readString("data");
        bean = loadComp(id);
        if (StringUtils.isEmpty(data)) {
            throw new SwException("没有待保存的数据！");
        }
        bean.setDataset(ModelFormHelper.buildSaveDataset(data));
        DbEngine.getInstance().doTrans(new AbsDbWorker() {
            @Override
            public void work() {
                EntityDao<ModelForm> dao = DbEngine.getInstance().findDao(tableName);

                ModelTable table = ModelTableCache.getInstance().getByName(tableName);
                ModelField field = table.findFieldByType(SwEnum.FieldType.UPDATE_USER.value);
                if (field != null) bean.put(field.getName(), us.getUserId());
                dao.updateEntity(bean, "mf_dataset");
            }

            @Override
            public void doAfterDbCommit() {
                saveSuccess();
            }

            @Override
            public void doAfterDbRollback() {
                saveFailed();
            }
        });
        return R.success();
    }

    public R saveModel() {
        long id = params.readLong("id");
        String data = params.readString("data");
        bean = loadComp(id);
        if (StringUtils.isEmpty(data)) {
            throw new SwException("没有待保存的数据！");
        }
        bean.setContent(data);
        bean.setContent(ModelFormHelper.buildSaveModel(bean));
        DbEngine.getInstance().doTrans(new AbsDbWorker() {
            @Override
            public void work() {
                EntityDao<ModelForm> dao = DbEngine.getInstance().findDao(tableName);

                ModelTable table = ModelTableCache.getInstance().getByName(tableName);
                ModelField field = table.findFieldByType(SwEnum.FieldType.UPDATE_USER.value);
                if (field != null) bean.put(field.getName(), us.getUserId());
                dao.updateEntity(bean, "mf_content");
            }

            @Override
            public void doAfterDbCommit() {
                saveSuccess();
            }

            @Override
            public void doAfterDbRollback() {
                saveFailed();
            }
        });
        return R.success();
    }
}
