package cc.smtweb.system.bpm.web.design.form.define;

import cc.smtweb.framework.core.common.SwException;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Akmm at 2022/4/18 14:20
 * 数据集定义
 */
public class PageDataset {
    //唯一标识
    public String id;
    //名称
    public String name;
    //中文名
    public String label;
    //类别：list-列表；form-表单；editList-编辑列表;tree
    public String type;
    //主表
    public long masterTable;
    //id字段 对应field的name
    public String idField;
    //list的自定义sql，仅列表支持，select fields from tables [condition] group by xxx order by xxx
    public String sql;
    //固定条件，如f1='a' and f2=:p1
    public String fixedCond;
    //是否懒加载
    public boolean lazy;
    //是否可编辑
    public boolean canEdit;
    //select的字段
    public List<PageDatasetField> fields;
    public List<PageDatasetFilter> filters;
    public List<PageDatasetEnumData> data;
    public PageDatasetDynCond dynCond;
    public List<PageDatasetSortField> sortFields;

    private Map<String, PageDatasetField> mapField;
    private Map<String, PageDatasetFilter> mapFilter;

    public PageDatasetField getField(String name) {
        if (mapField == null) resetFields();
        return mapField.get(name);
    }

    public PageDatasetFilter getFilter(String name) {
        if (mapFilter == null) resetFields();
        return mapFilter.get(name);
    }

    /**
     * 重置丰富字段信息
     */
    public void resetFields() {
        mapField = new HashMap<>();
        resetFields(mapField, fields);
        mapFilter = new HashMap<>();
        resetFields(mapFilter, filters);
        for (PageDatasetFilter filter: filters) {
            if (StringUtils.isEmpty(filter.sqlName)) {
                filter.sqlName = filter.field;
            }
        }
    }

    private <T extends PageDatasetField> void resetFields(Map<String, T> map, List<T> list) {
        ModelTableCache cache = ModelTableCache.getInstance();
        ModelTable table = null;
        for (T field: list) {
            map.put(field.name, field);
            if (field.table <= 0 || StringUtils.isEmpty(field.field)) continue;
            if (table == null || table.getId() != field.table) {
                table = cache.get(field.table);
            }
            if (table == null) continue;
            ModelField mf = table.findField(field.field);
            if (mf == null) throw new SwException("未找到表字段的定义信息（" + table.getName() + "." + field.field + "）");
            field.table_text = table.getTitle();
            field.fieldType = mf.getFieldType();
            field.notNull = mf.getNotNull();
            field.link = mf.getLink();

            if (StringUtils.isEmpty(field.label)) field.label = mf.getTitle();
            if (StringUtils.isEmpty(field.dataType)) field.dataType = mf.getDataType();
            if (StringUtils.isEmpty(field.remark)) field.remark = mf.getRemark();
            if (StringUtils.isEmpty(field.editor)) field.editor = mf.getEditor();
        }
    }

    public PageDatasetFilter findFilterByDs(String dsName) {
        for (PageDatasetFilter filter: filters) {
            if (SwEnum.FilterType.LINK.value.equals(filter.type) && dsName.equals(filter.linkDb)) {
                return filter;
            }
        }
        return null;
    }
}
