package cc.smtweb.system.bpm.web.design.table;

import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelCatalog;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.mvc.service.AbstractTreeHandler;
import cc.smtweb.framework.core.mvc.service.TreeHelper;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.CommUtil;
import cc.smtweb.system.bpm.web.design.form.ModelForm;
import cc.smtweb.system.bpm.web.design.form.ModelFormCache;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by Akmm at 2022/3/21 18:22
 */
public class ModelCatalogTreeHandler extends AbstractTreeHandler<DefaultEntity> {
    //查询类型：0-目录；1-表定义;2-页面；3-控件；23-页面和控件
    private final static int TYPE_CATALOG = 0;
    private final static int TYPE_TABLE = 1;
    private final static int TYPE_PAGE = 2;
    private final static int TYPE_WIDGET = 3;
    private final static int TYPE_FORM = 23;

    private long prj_id;//所属项目
    private int type;//查询类型：0-目录；1-表定义;2-页面定义

    private ModelCatalogTreeHelper mcTreeHelper = null;

    @Override
    public void init(SwMap params, UserSession us) {
        super.init(params, us);
        type = params.readInt("type");
        prj_id = params.readLong("prj_id");
        mcTreeHelper = (ModelCatalogTreeHelper) TreeHelper.getTreeHelper(ModelCatalog.ENTITY_NAME, ModelCatalogTreeHelper.class);
    }

    @Override
    protected List<DefaultEntity> getChildren(DefaultEntity bean) {
        if (bean instanceof ModelCatalog) {
            return super.getChildren(bean);
        }
        return null;
    }

    @Override
    protected List<DefaultEntity> filterData() {

        EntityDao<ModelCatalog> dao = DbEngine.getInstance().findDao(ModelCatalog.ENTITY_NAME);
        String text = "%" + params.readString("text") + "%";
        List<ModelCatalog> list = dao.queryWhere(" mc_prj_id=? and (mc_name like ? or mc_code like ?) order by mc_name", prj_id, text, text);
        List<DefaultEntity> listRet = new ArrayList<>(list);

        switch (type) {
            case TYPE_TABLE:
                EntityDao<ModelTable> tabledao = DbEngine.getInstance().findDao(ModelTable.class);
                List<ModelTable> l = tabledao.queryWhere(" tb_prj_id=? and (tb_name like ? or tb_title like ?) order by tb_name", prj_id, text, text);
                listRet.addAll(l);
                break;
            case TYPE_PAGE:
            case TYPE_WIDGET:
            case TYPE_FORM:
                EntityDao<ModelForm> formdao = DbEngine.getInstance().findDao(ModelForm.class);
                List<ModelForm> lf;
                if (type != TYPE_FORM) {
                    lf = formdao.queryWhere(" mf_prj_id=? and (mf_name like ? or mf_title like ?) order by tb_name", prj_id, text, text);
                } else {
                    //数据库type 0-页面 1-控件
                    lf = formdao.queryWhere(" mf_prj_id=? and mf_type=? and (mf_name like ? or mf_title like ?) order by tb_name", prj_id, type - 2, text, text);
                }
                listRet.addAll(lf);
        }
        return listRet;
    }

    @Override
    protected List<DefaultEntity> getChildren(long id) {
        List<ModelCatalog> list = mcTreeHelper.getChildren(id, prj_id, (o1, o2) -> CommUtil.chineseCompare(o1.getName(), o2.getName()));

        List<DefaultEntity> listRet = new ArrayList<>(list);

        switch (type) {
            case TYPE_TABLE:
                addTableChildren(listRet, id);
                break;
            case TYPE_PAGE:
            case TYPE_WIDGET:
            case TYPE_FORM:
                addPageChildren(listRet, id, type);
        }
        return listRet;
    }

    //增加表
    private void addTableChildren(List<DefaultEntity> listRet, long mcid) {
        Collection<ModelTable> set = ModelTableCache.getInstance().getTablesByMc(mcid, (o1, o2) -> CommUtil.chineseCompare(o1.getName(), o2.getName()));
        if (set == null || set.isEmpty()) return;

        listRet.addAll(set);
    }

    //增加页面定义
    private void addPageChildren(List<DefaultEntity> listRet, long mcid, int type) {
        List<ModelForm> set = ModelFormCache.getInstance().getFormsByMc(mcid, (o1, o2) -> {
            if (o1.getType() != o2.getType()) return o1.getType() - o2.getType();
            return CommUtil.chineseCompare(o1.getTitle(), o2.getTitle());
        });
        if (set == null || set.isEmpty()) return;
        if (type != TYPE_FORM) {
            int pt = type - 2;
            for (ModelForm form: set) {
                if (form.getType() == pt) {
                    listRet.add(form);
                }
            }
        } else {
            listRet.addAll(set);
        }
    }

    @Override
    protected long getId(DefaultEntity bean) {
        return bean.getEntityId();
    }

    @Override
    protected String getText(DefaultEntity bean) {
        if (bean instanceof ModelCatalog) {
            return ((ModelCatalog) bean).getName();
        }
        if (bean instanceof ModelTable) {
            return ((ModelTable) bean).getTitle();
        }

        if (bean instanceof ModelForm) {
            return ((ModelForm)bean).getTitle();
        }
        return null;
    }

    @Override
    protected void buildNode(SwMap node, DefaultEntity bean) {
        super.buildNode(node, bean);
        if (bean instanceof ModelCatalog) {
            node.put("type", TYPE_CATALOG);
        } else if (bean instanceof ModelTable) {
            node.put("type", TYPE_TABLE);
        } else if (bean instanceof ModelForm) {
            node.put("type", ((ModelForm)bean).getType() + 2);
        }
    }
}
