package cc.smtweb.system.bpm.web.engine.dynPage;

import cc.smtweb.framework.core.common.SwException;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.common.SwConsts;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.mvc.service.SqlNamedPara;
import cc.smtweb.framework.core.util.MapUtil;
import cc.smtweb.system.bpm.web.design.form.define.*;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import static cc.smtweb.framework.core.common.SwConsts.TOTAL_KEY;

/**
 * Created by Akmm at 2022/4/23 10:01
 * 动态页面辅助类
 */
public class DynPageHelper {
    /**
     * 新建bean
     *
     * @param dataSet
     * @return
     */
    public static SwMap createBean(PageDataset dataSet) {
        //主表
        ModelTable masterTable = ModelTableCache.getInstance().get(dataSet.masterTable);
        EntityDao<DefaultEntity> dao = DbEngine.getInstance().findDao(masterTable.getTableName());
        try {
            DefaultEntity bean = dao.createBean();
            return bean.getData();
        } catch (Exception e) {
            throw new SwException(e);
        }
    }

    /**
     * 构建select fields from table
     *
     * @param dataSet
     * @return
     */
    public static SqlNamedPara buildSelectSql(PageDataset dataSet, Map<String, Object> params) {
        StringBuilder sql = new StringBuilder(512);

        SqlNamedPara sqlNamedPara = buildWhereSql(dataSet, params);

        sql.append(buildSelFieldsSql(dataSet, sqlNamedPara));
        if (StringUtils.isNotEmpty(sqlNamedPara.sql)) {
            sql.append(" where ").append(sqlNamedPara.sql);
        }
        if (dataSet.sortFields != null) {
            String s = "";
            for (PageDatasetSortField sf : dataSet.sortFields) {
                s += "," + sf.field + " " + sf.type;
            }
            if (StringUtils.isNotEmpty(s)) {
                sql.append(" order by ").append(s.substring(1));
            }
        }
        sqlNamedPara.sql = sql.toString();
        sqlNamedPara.page = MapUtil.readInt(params, SwConsts.PARAM_PAGE);
        sqlNamedPara.rows = MapUtil.readInt(params, SwConsts.PARAM_ROWS);
        return sqlNamedPara;
    }

    public static SqlNamedPara buildSumSql(PageDataset dataSet, Map<String, Object> params) {
        SqlNamedPara sqlNamedPara = buildSelectSql(dataSet, params);
        StringBuilder sql = new StringBuilder(256);
        sql.append("select count(1) " + TOTAL_KEY);
        for (PageDatasetField field : dataSet.fields) {
            if (StringUtils.isEmpty(field.summary)) continue;
            sql.append(",");
            if (!SwEnum.SummaryType.instance.isText(field.summary)) {
                sql.append(field.summary).append("(").append(field.name).append(") ").append(field.name);
            } else {
                sql.append("'").append(field.summary).append("' ").append(field.name);
            }
        }
        sqlNamedPara.sql = sql.toString() + " from (" + sqlNamedPara.sql + ") xxxxa";
        return sqlNamedPara;
    }

    private static String buildSelFieldsSql(PageDataset dataSet, SqlNamedPara sqlNamedPara) {
        StringBuilder sql = new StringBuilder(512);
        //主表
        ModelTable masterTable = ModelTableCache.getInstance().get(dataSet.masterTable);
        //非查询列表，或sql为空，则自己组装select sql
        if (!SwEnum.DatasetType.LIST.equals(dataSet.type) || StringUtils.isEmpty(dataSet.sql)) {
            sql.append("select ");
            for (PageDatasetField field : dataSet.fields) {
                sql.append(field.field);
                //加别名
                if (!field.field.equalsIgnoreCase(field.name)) {
                    sql.append(" ").append(field.name);
                    sqlNamedPara.addFieldAlias(field.field, field.name);
                }
                sql.append(",");
            }
            sql.setCharAt(sql.length() - 1, ' ');
            sql.append(" from ").append(masterTable.getName());
            return sql.toString();
        } else {
            return dataSet.sql;
        }
    }

    /**
     * 构建where条件：组合固定和动态条件
     *
     * @param dataSet
     * @param params
     * @return
     */
    public static SqlNamedPara buildWhereSql(PageDataset dataSet, Map<String, Object> params) {
        StringBuilder sql = new StringBuilder(512);
        SwMap args = new SwMap();

        if (StringUtils.isNotEmpty(dataSet.fixedCond)) {
            sql.append("(").append(dataSet.fixedCond).append(")");
        }
        //记录归属于固定条件的filter，最后设置参数
        Set<String> setFixedFilter = new HashSet<>();
        for (PageDatasetFilter filter : dataSet.filters) {
            setFixedFilter.add(filter.name);
        }
        if (!dataSet.dynCond.isEmpty()) {
            String s = buildDynCondSql(dataSet, dataSet.dynCond, params, args, setFixedFilter);
            if (StringUtils.isNotEmpty(s)) {
                if (sql.length() > 0) sql.append(" and ");
                sql.append(s);
            }
        }
        for (String s : setFixedFilter) {
            args.put(s, params.get(s));
        }
        return new SqlNamedPara(sql.toString(), args);
    }

    /**
     * 构建动态条件
     *
     * @param dataSet
     * @param dynCond
     * @param params
     * @param args
     * @param setFixedFilter
     * @return
     */
    private static String buildDynCondSql(PageDataset dataSet, PageDatasetDynCond dynCond, Map<String, Object> params, SwMap args, Set<String> setFixedFilter) {
        if (dynCond.isCondOpt()) {//是and/or
            if (dynCond.children == null || dynCond.children.isEmpty()) return "";
            StringBuilder sql = new StringBuilder(256);
            boolean b = false;
            //递归调用
            for (PageDatasetDynCond dc : dynCond.children) {
                String s = buildDynCondSql(dataSet, dc, params, args, setFixedFilter);
                if (StringUtils.isEmpty(s)) continue;

                if (b) sql.append(" ").append(dynCond.opt).append(" ");
                sql.append(s);
                b = true;
            }
            if (sql.length() > 0) {
                return "(" + sql.toString() + ")";
            }
            return "";
        }

        boolean isNameSelf = setFixedFilter.contains(dynCond.param);
        setFixedFilter.remove(dynCond.param);

        PageDatasetFilter filter = dataSet.getFilter(dynCond.param);
        if (filter == null) throw new SwException("没有找到filter（" + dynCond.param + "）！");

        Object value = null;
        if (SwEnum.FilterType.CONST.value.equals(filter.type)) {
            value = filter.value;
            //todo 有变量的情况需要处理
        } else {
            value = params.get(filter.name);
        }
        if (value == null || StringUtils.isEmpty(value.toString())) {
            if (filter.required) {
                throw new SwException("过滤条件不能为空（" + dynCond.param + "）！");
            }
            return null;
        }
        IBuilderExpr builder = getBuilder(dynCond.opt);
        String ns = isNameSelf ? filter.name : filter.name + "_" + dynCond.hashCode();
        return builder.build(dynCond.opt, filter.sqlName, ns, value, args);
    }

    private static Map<String, IBuilderExpr> mapBuilder;
    private static IBuilderExpr baseBuilder;

    static {
        baseBuilder = (opt, field, name, value, args) -> {
            args.put(name, value);
            return field + " " + opt + " :" + name;
        };
        mapBuilder = new HashMap<>();
        mapBuilder.put(SwEnum.OptType.LIKE.value, (opt, field, name, value, args) -> {
            args.put(name, "%" + value + "%");
            return field + " like :" + name;
        });

        mapBuilder.put(SwEnum.OptType.PLIKE.value, (opt, field, name, value, args) -> {
            args.put(name, value + "%");
            return field + " like :" + name;
        });

        mapBuilder.put(SwEnum.OptType.BT.value, (opt, field, name, value, args) -> {
            String[] ss = value.toString().split(",");
            if (ss.length != 2) throw new SwException("介于条件，参数值个数错误！");
            args.put(name + "_1", ss[0]);
            args.put(name + "_2", ss[1]);
            return "(" + field + ">=:" + name + "_1 and " + field + "<=:" + name + "_2)";
        });
    }

    private static IBuilderExpr getBuilder(String opt) {
        IBuilderExpr builder = mapBuilder.get(opt);
        return builder != null ? builder : baseBuilder;
    }

    interface IBuilderExpr {
        String build(String opt, String field, String name, Object value, Map<String, Object> args);
    }

}
