package cc.smtweb.system.bpm.web.engine.dynPage;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityHelper;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.mvc.service.AbstractListHandler;
import cc.smtweb.framework.core.mvc.service.SqlNamedPara;
import cc.smtweb.framework.core.mvc.service.SqlPara;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;

import java.util.List;

import static cc.smtweb.framework.core.common.SwConsts.TOTAL_KEY;

/**
 * Created by Akmm at 2022/5/26 15:58
 */
public class DynPageListHandler extends AbstractListHandler {
    protected long pageId;
    //过滤条件
    private SwMap filter;

    //对应的数据集定义
    private PageDataset pageDataSet;

    public DynPageListHandler(long pageId, SwMap filter, PageDataset pageDataSet) {
        this.pageId = pageId;
        this.filter = filter;
        this.pageDataSet = pageDataSet;
    }

    @Override
    protected String getCompId() {
        return "dynpage_" + pageId + "_" + pageDataSet.id;
    }

    @Override
    protected String getPkFieldName() {
        return pageDataSet.idField;
    }

    @Override
    protected SqlPara buildSqlPara() {
        return DynPageHelper.buildSelectSql(pageDataSet, filter);
    }

    @Override
    protected List<SwMap> queryData(String sql, SqlPara sqlPara) {
        List<SwMap> list;
        SqlNamedPara sp = (SqlNamedPara)sqlPara;
        if (sqlPara.page > 0 && sqlPara.rows > 0) {
            list = DbEngine.getInstance().pagedQueryN(sql, SwMap.class, (sqlPara.page - 1) * sqlPara.rows, sqlPara.rows, sp.mapParas);
        } else {
            list = DbEngine.getInstance().queryN(sql, sp.mapParas, SwMap.class);
        }
        ModelTable masterTable = ModelTableCache.getInstance().get(pageDataSet.masterTable);
        EntityHelper.loadBeanLink(masterTable.getName(), list, sp.mapFieldAlias);
        return list;
    }

    @Override
    public R getTotal() {
        SqlNamedPara sqlPara = DynPageHelper.buildSumSql(pageDataSet, filter);

        SwMap mapFooter = DbEngine.getInstance().queryEntityN(sqlPara.sql, sqlPara.mapParas, SwMap.class);

        R r = R.success();
        r.put("total", mapFooter.get(TOTAL_KEY));
        r.put("footer", mapFooter);
        return r;
    }
}
