package cc.smtweb.system.bpm.web.engine.dynPage;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwException;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityHelper;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.mvc.service.SqlNamedPara;
import cc.smtweb.framework.core.mvc.service.SwListData;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;

import java.util.List;

import static cc.smtweb.framework.core.common.SwConsts.TOTAL_KEY;

/**
 * Created by Akmm at 2022/4/21 17:53
 */
public class DynPageLoadHandler extends AbstractDynPageHandler {
    //数据集
    private String dbName;
    //过滤条件
    private SwMap filter = new SwMap();

    //对应的数据集定义
    private PageDataset pageDataSet;

    //列表工具类
    public DynPageListHandler getListWorker(SwMap filter, PageDataset pageDataSet) {
        DynPageListHandler listHandler = new DynPageListHandler(pageId, filter, pageDataSet);
        listHandler.init(params, us);
        return listHandler;
    }

    //新增操作，初始化定义的数据集
    public R add() {
        //返回的数据，以dataset.name为key，查出的结果（bean或list）为value
        SwMap mapRet = new SwMap();
        for (PageDataset dataSet : datasets.list) {
            if (!dataSet.canEdit) {
                continue;
            }
            if (SwEnum.DatasetType.FORM.value.equals(dataSet.type)) {//单表
                //懒加载，给个空对象
                if (dataSet.lazy) {
                    mapRet.put(dataSet.name, new SwMap());
                    continue;
                }
                mapRet.put(dataSet.name, DynPageHelper.createBean(dataSet));
            } else if (SwEnum.DatasetType.TREE.value.equals(dataSet.type)) {//树
                mapRet.put(dataSet.name, new SwMap());
            } else if (!SwEnum.DatasetType.ENUM.value.equals(dataSet.type)) {//非枚举
                mapRet.put(dataSet.name, SwListData.create(null, 0));
            }
        }
        return R.success(mapRet);
    }

    //新增操作，初始化定义的数据集
    public R addOne() {
        //对应的数据集定义
        PageDataset pageDataSet = readParamDs();

        if (!pageDataSet.canEdit) {
            return R.error("指定数据集为只读数据集[" + pageDataSet.label + "]！");
        }
        //懒加载，给个空对象
        return R.success(DynPageHelper.createBean(pageDataSet));
    }

    public R loadOne() {
        //过滤条件
        SwMap filter = params.readMap("filter");
        //对应的数据集定义
        PageDataset pageDataSet = readParamDs();

        DynRetBean bean = null;
        if (SwEnum.DatasetType.LIST.value.equals(pageDataSet.type)) {//列表类
            bean = DynRetBean.createList(getListWorker(filter, pageDataSet).buildListData());
        } else if (SwEnum.DatasetType.TREE.value.equals(pageDataSet.type)) {//树类
//            return new DynRetBean(loadTree());
        } else {//列表类
            bean = DynRetBean.createBean(provider.loadData(filter, pageDataSet));
        }

        return R.success(bean);
    }

    /**
     * 计算分页数据
     *
     * @return
     */
    public R getTotal() {
        //数据集
        PageDataset pageDataSet = readParamDs();
        //过滤条件
        SwMap filter = params.readMap("filter");

        return getListWorker(filter, pageDataSet).getTotal();
    }
}
