package cc.smtweb.system.bpm.web.task.service;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.annotation.SwBody;
import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.system.bpm.core.exception.BpmException;
import cc.smtweb.system.bpm.engine.process.impl.entity.LoadTaskVO;
import cc.smtweb.system.bpm.engine.process.impl.entity.NodeUsersVO;
import cc.smtweb.system.bpm.engine.process.runtime.Route;
import cc.smtweb.system.bpm.engine.process.runtime.RouteInfo;
import cc.smtweb.system.bpm.engine.process.task.Task;
import cc.smtweb.system.bpm.engine.process.vo.FlowAction;
import cc.smtweb.system.bpm.engine.process.vo.FlowNode;
import cc.smtweb.system.bpm.engine.process.vo.FlowNodeUser;
import cc.smtweb.system.bpm.spring.IProcessEngine;
import cc.smtweb.system.bpm.web.task.entity.UserGroupTargetPO;
import cc.smtweb.system.bpm.web.task.entity.UserPO;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//// "工作流任务接口"
@SwService
public class BpmTaskUserListService {
  @SwParam
  private DbEngine dbEngine;

  @SwParam
  private IProcessEngine processEngine;

//  @ApiOperation(value = "读取执行候选人名单")
  public R list(@SwBody LoadTaskVO loadTaskVO) {
    Task task = processEngine.loadTask(loadTaskVO.getTaskId());
    if (task == null) {
      return R.error("加载任务失败");
    }

    FlowAction flowAction = task.getProcessInstance().checkAndGetCurNodeAction(loadTaskVO.getActionKey());
    NodeUsersVO data = null;

    switch (flowAction.getType()) {
      case FlowAction.TYPE_COMMIT:
        FlowNode toFlowNode = task.getProcessInstance().getFlow().checkAndGetNode(loadTaskVO.getActionKey());
        data = loadUser(task, flowAction, toFlowNode);
        break;
      case FlowAction.TYPE_TRANSMIT:
        data = loadUser(task, flowAction, task.getProcessInstance().getCurNode());
        break;
      case FlowAction.TYPE_RETURN:
        data = loadReturnUser(task);
        break;
      default:
        break;
    }

    if (data != null) {
      data.setActionKey(loadTaskVO.getActionKey());
      data.setNodeKey(loadTaskVO.getToNodeKey());

      return R.success(data);
    }

    return R.error();
  }

  private NodeUsersVO loadUser(Task task, FlowAction flowAction, FlowNode toFlowNode) {
    NodeUsersVO data = new NodeUsersVO();

    RouteInfo nextProcessInfo = processEngine.routeCommit(task, toFlowNode);

    for (Route route: nextProcessInfo.getRoutes()) {
      List<UserPO> users = loadUsers(task.getProcessInstance().getId(), flowAction, route.getNode());
      data.add(toFlowNode, users);
    }

    return data;
  }

  private List<UserPO> loadUsers(long processId, FlowAction flowAction, FlowNode toFlowNode) {
    // toFlowNode.getUsers();
    // 用户组 + 过滤条件
    List<FlowNodeUser> flowNodeUsers = toFlowNode.getUsers();
    if (flowNodeUsers == null) {
      throw new BpmException("节点未设置用户组, id=" + toFlowNode.getId());
    }

    StringBuilder ids = new StringBuilder("(");
    for (FlowNodeUser flowNodeUser : flowNodeUsers) {
      if (flowNodeUser.getType() == FlowNodeUser.TYPE_USER_GROUP) {
        ids.append(flowNodeUser.getTarget()).append(",");
      }
    }

    List<UserPO> data = new ArrayList<>();
    Map<Long, UserPO> userMap = new HashMap<>();

    if (ids.length() > 1) {
      ids.setCharAt(ids.length() - 1, ')');
      List<UserGroupTargetPO> userGroupTargetPos = dbEngine.query("SELECT ugt_id,ugt_type,ugt_target FROM sw_user.sys_user_group_target WHERE ugt_ug_id in " + ids.toString(), UserGroupTargetPO.class);

      for (UserGroupTargetPO userGroupTargetPo : userGroupTargetPos) {
        List<UserPO> users = null;
        switch (userGroupTargetPo.getUgtType()) {
          case FlowNodeUser.TYPE_ROLE:
            users = dbEngine.query("SELECT user_id, user_name, user_status FROM sw_user.sys_user_role LEFT JOIN smt_user.sys_user ON(ur_user_id=user_id) WHERE ur_role_id in (" + userGroupTargetPo.getUgtTarget() + ")", UserPO.class);
            break;
          case FlowNodeUser.TYPE_USER:
            users = dbEngine.query("select user_id, user_name, user_status from sw_user.sys_user where user_id in (" + userGroupTargetPo.getUgtTarget() + ")", UserPO.class);
            break;
          case FlowNodeUser.TYPE_DEPT:
            users = dbEngine.query("select user_id, user_name, user_status from sw_user.sys_user where user_dept_id in (" + userGroupTargetPo.getUgtTarget() + ")", UserPO.class);
            break;
          default:
            break;
        }

        if (users != null) {
          for (UserPO userPo : users) {
            // 去除重复
            if (userMap.putIfAbsent(userPo.getUserId(), userPo) == null) {
              data.add(userPo);
            }
          }
        }
      }
    }

    return data;
  }

  private NodeUsersVO loadReturnUser(Task task) {
    RouteInfo nextProcessInfo = processEngine.routeReturn(task);

    NodeUsersVO data = new NodeUsersVO();

    for (Route route: nextProcessInfo.getRoutes()) {
      Long userId = dbEngine.queryLong("SELECT tl_assignee_uid FROM sw_bpm.bpm_task_log WHERE tl_process_id=? AND tl_node_key=? ORDER BY tl_create_time desc LIMIT 1",
              task.getProcessInstance().getId(), route.getNode().getId());

      if (userId != null) {
        List<UserPO> users = dbEngine.query("select user_id, user_name, user_status from sw_user.sys_user where user_id=?",
                UserPO.class, userId);

        data.add(route.getNode(), users);
      }
    }

    return data;
  }
}
