package cc.smtweb.system.bpm.web.ui.dao;

import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.annotation.SwBean;
import cc.smtweb.framework.core.util.JsonUtil;
import cc.smtweb.system.bpm.core.ui.BpmPageContext;
import cc.smtweb.system.bpm.core.ui.builder.TreeDataBuilder;
import cc.smtweb.system.bpm.core.ui.entity.dataset.BpmDataset;
import cc.smtweb.system.bpm.engine.ui.entity.po.DynFormLogPO;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

@SwBean
public class JsonDatasetTreeDao extends AbstractJsonDatasetPlugin {
  public List<SwMap> treeAll(BpmPageContext ctx) {
    BpmDataset dataset = ctx.getDataset();
    TreeDataBuilder treeDataBuilder = new TreeDataBuilder(dataset, null, null);

    List<DynFormLogPO> poList = dbEngine.query("select dfl_id, dfl_content from sw_bpm.bpm_dyn_form_log where dfl_table=?",
            DynFormLogPO.class, dataset.fullName());

    List<SwMap> list = new ArrayList<>();

    for (DynFormLogPO po: poList) {
      list.add(treeDataBuilder.mapTreeFields(dataset, po));
    }

    return treeDataBuilder.buildTree(list, true);
  }

  public List<SwMap> tree(BpmPageContext ctx, Long parentId) {
    BpmDataset dataset = ctx.getDataset();
    TreeDataBuilder treeDataBuilder = new TreeDataBuilder(dataset, parentId, null);

    List<DynFormLogPO> poList = dbEngine.query("select dfl_id, dfl_content from sw_bpm.bpm_dyn_form_log where dfl_table=? and " +
                    "JSON_EXTRACT(dfl_content, '$." + treeDataBuilder.getParentField().getFieldName() + "')=?",
            DynFormLogPO.class, dataset.fullName(), parentId);

    List<SwMap> list = new ArrayList<>();

    for (DynFormLogPO po: poList) {
      list.add(treeDataBuilder.mapTreeFields(dataset, po));
    }

    return list;
  }

  public int move(BpmPageContext ctx, TreeDataBuilder treeDataBuilder) {
    Long parentId = treeDataBuilder.getParentId();
    Long[] orders = treeDataBuilder.getOrders();

    String tableName = treeDataBuilder.getDataset().fullName();
    List<DynFormLogPO> poList = dbEngine.query("select dfl_id, dfl_content from sw_bpm.bpm_dyn_form_log where dfl_table=? and dfl_id in ("
                    + StringUtils.join(orders, ",") + ")",
            DynFormLogPO.class, tableName);

    int count = 0;

    for (DynFormLogPO po: poList) {
      SwMap content = treeDataBuilder.updateOrder(po, parentId, findOrderValue(orders, po.getDflId()));
      int ret = dbEngine.update("update sw_bpm.bpm_dyn_form_log set dfl_content=? where dfl_id=? and dfl_table=?",
              JsonUtil.encodeString(content), po.getDflId(), tableName);
      if (ret > 0) {
        count++;
      }
    }

    return count;
  }

  private static int findOrderValue(Long[] orders, Long id) {
    for (int i = 0; i < orders.length; i++) {
      if (id.equals(orders[i])) {
        return i + 1;
      }
    }

    return 0;
  }
}
